
#ifndef __EXTSUBCH_HPP__
#define __EXTSUBCH_HPP__

#include "DAS_info.hpp"
#include <vector>
#include <string>
#include <iostream>
#include <cstdio>
#include <cstdlib>
#include <cmath>
// using namespace std;

extern "C"{
#include <stdint.h>
}

class ExtSubCh {
public:
  enum mode_t{Mk4GeoVSI, Normal};
private:
  const uint8_t* _rawarr;
  const uint8_t* _ptr;
  int32_t _arr_size; ///< array size (byte)
  const uint8_t* _ptr_end;
  int32_t _remain;
  vector <uint8_t>::iterator _iptr;
  const DAS_Info* _indas;
  DAS_Info _outdas;
  uint8_t _transTbl[256]; ///< Table for 1bit extraction 
  void TableGen();
  void TableGen2();
  mode_t  mode_type; 
public:
  /// Constructor 
  ExtSubCh();
  ExtSubCh(const uint8_t* a, const int32_t size, const DAS_Info* idas, 
	   const DAS_Info& odas ):
    _rawarr(a), _ptr(a), _arr_size(size), _ptr_end(a+_arr_size),
    _remain(size), _indas(idas), _outdas(odas), mode_type(Normal){ ; }  
  void Setup(const uint8_t* a, const int32_t size, const DAS_Info* idas, const DAS_Info& odas );
  /// Set Output DAS-Information 
  void Set_OutDas(const DAS_Info& das){_outdas = das;}
//   /*! Extracting data to *ptr by <size> amount 
//    * @param *ptr Point for output extracted data.
//    * @param size Data Size to be extracted.
//    * @return  
//    *   - -1: Light error, to be try again
//    *   - -2: Serious error, to be returned without trying again.
//    */ 
//   int Extract(uint8_t* ptr, const int32_t size);
  /*! Extracting data to ptr by size amount 
   * @param ptr Reference to vector array <uint8_t> for output extracted data.
   * @param size Data Size to be extracted.
   * @return  
   *   - -1: Light error, to be try again
   *   - -2: Serious error, to be returned without trying again.
   */ 
  int Extract(vector <uint8_t>& ptr, const int32_t size);
  int Extract1bit(vector <uint8_t>& ptr, const int32_t size);
  void Set_Mode(const mode_t a){mode_type = a;}
private:
  // Extracting Subchannels, pointer version
  int Ext32_2( uint8_t* arr, const int32_t npt);
  int Ext32_4( uint8_t* arr, const int32_t npt);
  int Ext32_8( uint8_t* arr, const int32_t npt);
  int Ext32_16( uint8_t* arr,const int32_t npt);
  int Ext32_2Mk4geo( uint8_t* arr, const int32_t npt);
  int Ext32_4Mk4geo( uint8_t* arr, const int32_t npt);
  int Ext32_8Mk4geo( uint8_t* arr, const int32_t npt);
  int Ext32_16Mk4geo( uint8_t* arr,const int32_t npt);
  int Ext32_32Mk4geo( uint8_t* arr,const int32_t npt);
  int Ext32_2to1bit2( uint8_t* arr, const int32_t npt);
  int Ext32_2to1bit4( uint8_t* arr, const int32_t npt);
  int Ext32_2to1bit8( uint8_t* arr, const int32_t npt);
  int Ext32_2to1bit16( uint8_t* arr,const int32_t npt);
  int Ext32_2to1bit2Mk4geo( uint8_t* arr, const int32_t npt);
  int Ext32_2to1bit4Mk4geo( uint8_t* arr, const int32_t npt);
  int Ext32_2to1bit8Mk4geo( uint8_t* arr, const int32_t npt);
  int Ext32_2to1bit16Mk4geo( uint8_t* arr,const int32_t npt);
  int Ext_identical32( uint8_t* arr, const int32_t npt);
  int Ext_identical16( uint8_t* arr, const int32_t npt);
  int Ext_identical8( uint8_t* arr, const int32_t npt);
  int Ext_identical4( uint8_t* arr, const int32_t npt);
  int Ext_identical2( uint8_t* arr, const int32_t npt);
  int Ext_identical1( uint8_t* arr, const int32_t npt);
  int Ext16_1( uint8_t* arr, const int32_t npt);
  int Ext16_2( uint8_t* arr, const int32_t npt);
  int Ext16_4( uint8_t* arr, const int32_t npt);
  int Ext16_4Mk4geo( uint8_t* arr, const int32_t npt);
  int Ext16_8( uint8_t* arr, const int32_t npt);
  int Ext16_2to1bit2( uint8_t* arr, const int32_t npt);
  int Ext16_2to1bit4( uint8_t* arr, const int32_t npt);
  int Ext16_2to1bit8( uint8_t* arr, const int32_t npt);
  int Ext8_1( uint8_t* arr, const int32_t npt);
  int Ext8_2( uint8_t* arr, const int32_t npt);
  int Ext8_4( uint8_t* arr, const int32_t npt);
  int Ext8_2to1bit2( uint8_t* arr, const int32_t npt);
  int Ext8_2to1bit4( uint8_t* arr, const int32_t npt);
  int Ext4_1( uint8_t* arr, const int32_t npt);
  int Ext4_2( uint8_t* arr, const int32_t npt);
  int Ext2_1( uint8_t* arr, const int32_t npt);
};
 

#endif  // end of __EXTSUBCH_HPP__
