/**  Input Device abstract Model
 * Abstract class for Input device (Device:K5/VSI, K5/VSSP, Network, File).
 * $Id: InDev_mdl.hpp,v 1.2 2010/01/02 03:29:17 sekido Exp $
 */


#ifndef __INDEV_MDL_HPP__
#define __INDEV_MDL_HPP__

#include <vector>
#include <iostream>
#include <string>
#include <ctime>
using namespace std;


#include "DataPage.hpp"
#include "VLBI_Frame.hpp"
#include "DAS_info.hpp"
#include "Thread.h"

extern "C" {
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
}

/** Input Device Model Abstract Class
 * $Id: InDev_mdl.hpp,v 1.2 2010/01/02 03:29:17 sekido Exp $
 */
class InDev_Model:public Thread{
public:
  /// Return types of Input device
  enum IndevReturns_t {DATA_END, GEN_ERROR, GEN_WARNING, NORMAL};
  /// Types of input device 
  enum  InputModelType_t {FILE, NET, K5VSI, K5VSSP, MEM};
  enum  InputState_t {READY, STOP};
  //**** Function ********
  InDev_Model():vlbi_frame(0),page_wait(-1){;} ///< Constructor
  /// Destructor 
  virtual ~InDev_Model(){ cerr << "InDev_Model:: Destructor runs.\n";}
  virtual const int OpenDev()=0; ///< Abstract Open device/file 
  virtual const int CloseDev()=0; ///< Absract Open device/file 
  virtual const int TestDev()=0; ///< Test device/file Permission  
  //  /** Abstract func of Mapping Buffer address */
  //  virtual const IndevReturns_t MappingData(vector <uint8_t>& pntr, const time_t epoch)=0;
  /** Abstract func of Release Mapped Buffer address */
  virtual const IndevReturns_t UnMapping(const time_t data_epoch)=0;
  /// Mapping buffer address and Returns pointer address
  virtual const uint8_t* MappingData(time_t epoch)=0; 
  /**  Abstract function to return page_size(1sec) in 4-byte unit  */
  virtual uint32_t PageLength()=0;   
  /// Abstract function to retuen Device type in string
  virtual string StrDevType()=0;
  /// Returns Device type 
  virtual InputModelType_t DevType()=0;
  /** Abstract function to Setup DAS informaiton */
  virtual int Set_DasInfo(const DAS_Info& inf);
  /// Abstract funciton to initialize data pages
  virtual int InitPage()=0; ///< Initialize Pages
  /// Abstract function to return Epoch
  virtual time_t Epoch(const int i)const =0;
  /// Abstract function to reaturn page number of the earliest epoch
  virtual int FirstPage()  =0;
  /// Abstract function to return Ready or not
  virtual const bool Ready();
  /// Abstract function to return page state in string 
  virtual string StrPageState(const int lvl=0)=0;
  /** Set Page wait time */
  void Set_PageWait(const int sec){page_wait =sec;}
  /// Set pointer of VLBI Frame class
  virtual void Set_VlbiFrame(VLBI_Frame *vf){vlbi_frame = vf; }
  const DAS_Info* Das_inf() const; ///< Returns pointer of DAS info
  /// Returns VLBI Frame Type in number 
  VLBI_Frame::VLBI_Frame_t VlbiFrameType()const;
  /// Return Framesize in Bytes
  int32_t Framesize()const;
  /// Abstract Function to indicate Page is End 
  virtual void PageEnd()=0;
  virtual void Stop(){inputState=STOP;}  ///< Stop Rxing and Rate monitoring.
protected:
  VLBI_Frame *vlbi_frame; ///< VLBI_frame to Interprete data from device/file
  //    string infile ; ///< input file name
    int  page_wait; ///< page waiting limit in sec. -1: endless 
  //  Mutex mutex_page; ///< mutex use to waiting for page becoming ready.
  InputState_t  inputState; ///< Input model State (READY or STOP)
private:
  InDev_Model(InDev_Model &mdl);///< Copy Constructor prohibited
public:
};

// inline void InDev_Model::Stop(){

//   if ( Status() == Thread::RUNNING ) {
//     //    cancel();
//     rcv_protocol->protocol_Stop();
//     join();
//     Logsystem::error_logger()->Tstamp()  << " input_mdl joined.\n";
//   }
//   InitPage(); // Initialize pages for next use 
//   StopState();
// }


#endif
