
#include "InMem_mdl.hpp"

/** Input Memory Device model =******************************************
 *
 **/


void* InMem_Model::thread_func(){
  
  MemPageUpdate();

  CloseDev();
  StopState();
//  Thread::state = Thread::ENDING;
  cerr << " Leaving InFile_mdl-thread..." << endl;

  //  Ready(); // For next calling 

  return 0;
}


int InMem_Model::MemPageUpdate(){
  
  struct timeval tv;
  gettimeofday(&tv,0);
  double t1 = (double)tv.tv_sec + (double)tv.tv_usec/1000000.0;
  double t2 = t1 + time_span;
  time_t epoch = (time_t)t1;
  int ipage = epoch % dpage.Npage();
  while(dpage.IpageWriting() != ipage && dpage.Normal() ) {
    Logsystem::info_logger()->Tstamp()
      << "I InMem_mdl::MemPageUpdate(): Filling EMPTY at " 
      << dpage.IpageWriting() << "pg unitl " << ipage << "pg\n";
    dpage.WStepPage(DataPage::EMPTY);
  }

  dpage.SetTimeWritingPage(epoch);
  while (t2 > t1 && inputState != InDev_Model::STOP){
    int ret = dpage.WStepPage(DataPage::RD_OK); // Step to next page 
    if( ret < 0 ){ 	 
      Logsystem::debug_logger()->Tstamp() 
	<< "Waiting for Writing..Page[" << ipage << "](" 
	<< Epoch2Str(dpage.Epoch(ipage)) << ")"
	<< dpage.StrPageState()  << "\n";
      //      mutex_page.cond_timewait_2(1000); // 1sec 
      usleep(1000000); // 1sec 
    }else{
      epoch ++;
      dpage.SetTimeWritingPage(epoch);
      //      mutex_page.cond_signal();
    }
    gettimeofday(&tv,0);
    t1 = (double)tv.tv_sec + (double)tv.tv_usec/1000000.0;
  }
  ipage = epoch % dpage.Npage();
  dpage.ChgPageState(ipage, DataPage::DATA_END);
  return 0;
}



int InMem_Model::InitPage(){
  if(vlbi_frame ==0){
    cerr << "Error in void InMem_Model::InitPage(): vlbi_frame is empty."
	 << endl;
    return -1;
  }
  DAS_Info dasinfo =  vlbi_frame->DAS_Inf();
  uint32_t pagesize = dasinfo.Speed()*4;
  dpage.PageResize(pagesize);
  dpage.InitPages();
  return 0;
}

//vector <uint8_t>& InMem_Model::MappingData(const time_t epoch){
// const InMem_Model::IndevReturns_t 
// InMem_Model::MappingData(vector <uint8_t>& pntr, const time_t epoch){
//   int ipage = epoch % dpage.npage;
//   if(dpage.status[ipage] == DataPage::DATA_END){
//     cerr << "InFile_Model::MappingData(): END Found\n";
//     return DATA_END;
//   }
//   time_t t1 = dpage.Epoch(ipage);
//   int loop_cnt = N_LOOP_LIMIT;
//   char buf[2][256];
//   struct tm tm1, tm2;
//   while(t1 != epoch && loop_cnt > 0){
//     gmtime_r( &epoch, &tm1 );
//     gmtime_r( &t1, &tm2 );
//     strftime(buf[0],256,"%Y/%j-%H:%M:%S",&tm1 );
//     strftime(buf[1],256,"%Y/%j-%H:%M:%S",&tm2);
//     fprintf(stderr," @Mapping(): Waiting %d for %s but actually %s\n", 
// 	   loop_cnt, buf[0], buf[1]);
//     usleep(200000);
//     loop_cnt--;
//     t1 = dpage.Epoch(ipage);
//   }

//   if(t1 != epoch ){
//     fprintf(stderr,"Warining in InFile_Model::MappingData(): Time of page(%s) != requested(%s)\n",
// 	    buf[1],buf[0]);
//     return GEN_ERROR;
//   }
  
//   pntr =   dpage.dat[ipage];
//   return NORMAL;

// }


const uint8_t* InMem_Model::MappingData(time_t epoch){
  int ipage = epoch % dpage.Npage();
  if( dpage.TestPage(ipage, DataPage::DATA_END) ){
    cerr << "InFile_Model::MappingData(): END Found\n";
    return 0;
  }
  time_t t1 = dpage.Epoch(ipage);
  int loop_cnt = N_LOOP_LIMIT;

  char buf[2][256];
  struct tm tm1, tm2;
  while(t1 != epoch && loop_cnt > 0){
    gmtime_r( &epoch, &tm1 );
    gmtime_r( &t1, &tm2 );
    strftime(buf[0],256,"%Y/%j-%H:%M:%S",&tm1 );
    strftime(buf[1],256,"%Y/%j-%H:%M:%S",&tm2);
    fprintf(stderr," @Mapping(): Waiting %d for %s but actually %s\n", 
	    loop_cnt, buf[0],buf[1]);
    usleep(500000);
    loop_cnt--;
    t1 = dpage.Epoch(ipage);
  }
  
  if( t1 != epoch ){
    fprintf(stderr,"Error @InFile_Model::MappingData(): Time of page(%s) != requested(%s)\n",
	    buf[1],buf[0]);
    return 0;
  }

  //  return &(dpage.dat[ipage][0]);
  return dpage.Data(ipage);
}


const InMem_Model::IndevReturns_t 
InMem_Model::UnMapping(const time_t data_epoch){
  int ipage = data_epoch % dpage.Npage();
  time_t t1 = dpage.Epoch(ipage);
  IndevReturns_t ret = NORMAL;
  if(t1 != data_epoch){
    char buf[2][256];
    struct tm tm1, tm2;
    gmtime_r( &data_epoch, &tm1 );
    gmtime_r( &t1, &tm2 );
    strftime( buf[0], 256, "%Y/%j-%H:%M:%S", &tm1 );
    strftime( buf[1], 256, "%Y/%j-%H:%M:%S", &tm2 );
    fprintf(stderr, "Warining in InFile_Model::UnMapping: data epoch at page(%d) %s but actually %s \n",
	    ipage, buf[0],buf[1]);
    ret = GEN_WARNING;
  }
  dpage.ChgPageState(ipage, DataPage::WR_OK);
  return ret;
}

const int InMem_Model::OpenDev(){
  //  uint32_t byte_sec = vlbi_frame->DAS_Inf()->Speed() *4;
  //  dpage.Set_Npage(4);
  return 0;
}


void InMem_Model::PageEnd(){
  dpage.PageEnd();
}
