
#ifndef LINUX
#define LINUX
#endif
#ifndef _LARGEFILE_SOURCE 
#define _LARGEFILE_SOURCE
#endif 
#define _LARGEFILE64_SOURCE


#include "InVSI_mdl.hpp"

/** Constructor */
K5_VSI::K5_VSI():
  device(0), mem(0), device_epoch(0),
  vsi_devname("/dev/k5vsi"), mem_devname("/dev/mem"),
  npage(0), igrp(0), page_ptr(0),
  rpage(0), map_enable(false)
{
  make_tab(enc_table); 
  vlbi_frame = new K5VSI_Frame;
}

/** Destructor */
K5_VSI::~K5_VSI(){
  if(device!=0) CloseDev();
  //  if( vlbi_frame != 0){ delete vlbi_frame; vlbi_frame=0;}
}

const int K5_VSI::CloseDev(){
  if(device != 0) close(device);
  if(mem != 0) close(mem);
  device = 0;
  mem=0;
  ioctl(device,SET_FRAME_FREEZE,0x0);
  return 0;
}

const int K5_VSI::TestDev() {
  if((device=open(vsi_devname.c_str(), O_RDWR))==-1) {
    fprintf(stderr,"Error: device file [%s] open error\n",vsi_devname.c_str());
    return -1;}
  close(device);
  if(( mem=open(mem_devname.c_str() ,O_RDONLY))==-1) {
    perror("mem_device error:");
    fprintf(stderr,"device file [%s] open error\n",mem_devname.c_str());
    return -1;}
  close(mem);
  return 0;
}


const int K5_VSI::OpenDev(){
  if((device=open(vsi_devname.c_str(), O_RDWR))==-1) {
    fprintf(stderr,"Error: device file [%s] open error\n",vsi_devname.c_str());
    return -1;}
  if(( mem=open(mem_devname.c_str() ,O_RDONLY))==-1) {
    perror("mem_device error:");
    fprintf(stderr,"device file [%s] open error\n",mem_devname.c_str());
    return -1;}
  
  if(ioctl(device,SET_FRAME_FREEZE,0x7FFFFFFF) != 0){
    cerr << "Warning in DevOpen(): non-stopping DMA ioctl(SET_FRAME_FREEZE) failed.\n";
  }
  device_epoch = ioctl(device,GET_DEVICE_EPOCH,NULL);
  if(device_epoch ==-1){
    cerr << "Error in DevOpen(): GET_DEVICE_EPOCH failed\n";
    return -1;
  }
  char buf[80];
  strftime(buf,80,"%Y/%j-%H:%M:%S", gmtime(&device_epoch));

  struct timeval st;
  gettimeofday(&st,NULL);
  int tdif = abs(st.tv_sec - device_epoch);
  if( tdif > 10){
    cerr <<  "Error in DevOpen(): device time(" << buf
	 << " =" << device_epoch << ") differs from PC time("
	 << st.tv_sec << ") by " << tdif << "sec\n";
    return -1;
  }
  fprintf(stderr,"\tGet_Device time= %s at Device Open\n", buf);
  
  npage = ioctl(device,GET_FRAME_NUMBER,NULL);
  map_enable = true;
  return 0;
}


const int32_t K5_VSI::Srate()const {
  if( vlbi_frame == 0  ){
    cerr << "Error in K5_VSI::Srate(): *vlbi_frame is empty." << endl;
    return -1;
  }
  return vlbi_frame->DAS_Inf().Srate();
} 


const int K5_VSI::Nch()const{
  if( vlbi_frame == 0 ){
    cerr << "Error in K5_VSI::Nch(): *vlbi_frame is empty." << endl;
    return -1;
  }
  return vlbi_frame->DAS_Inf().Nch();
} 


const int K5_VSI::Nbit()const {
  if( vlbi_frame == 0){
    cerr << "Error in K5_VSI::Nbit(): *vlbi_frame is empty." << endl;
    return -1;
  }
  return vlbi_frame->DAS_Inf().Qbit();
}

const int K5_VSI::Set_Mode(const char* a){
  if( vlbi_frame == 0 ){
    cerr << "Error in K5_VSI::Set_Mode(): *vlbi_frame is empty." << endl;
    return -1;
  }
  int32_t srate(0);
  int nch(0);
  string tmp =a;
  int okflag=0;
  if(tmp.find("64Msps/2bit/16ch")!= tmp.npos) {srate=64000000;nch=16;okflag=1;}
  else if(tmp.find("32Msps/2bit/16ch")!= tmp.npos) {srate=32000000;nch=16;okflag=1;}
  else if(tmp.find("16Msps/2bit/16ch")!= tmp.npos) {srate=16000000;nch=16;okflag=1;}
  else if(tmp.find("08Msps/2bit/16ch")!= tmp.npos) {srate= 8000000;nch=16;okflag=1;}
  else if(tmp.find("04Msps/2bit/16ch")!= tmp.npos) {srate= 4000000;nch=16;okflag=1;}
  else if(tmp.find("64Msps/2bit/08ch")!= tmp.npos) {srate=64000000;nch=8;okflag=1;}
  else if(tmp.find("32Msps/2bit/08ch")!= tmp.npos) {srate=32000000;nch=8;okflag=1;}
  else if(tmp.find("16Msps/2bit/08ch")!= tmp.npos) {srate=16000000;nch=8;okflag=1;}
  else if(tmp.find("08Msps/2bit/08ch")!= tmp.npos) {srate= 8000000;nch=8;okflag=1;}
  else if(tmp.find("04Msps/2bit/08ch")!= tmp.npos) {srate= 4000000;nch=8;okflag=1;}
  else if(tmp.find("64Msps/2bit/04ch")!= tmp.npos) {srate=64000000;nch=8;okflag=1;}
  else if(tmp.find("32Msps/2bit/04ch")!= tmp.npos) {srate=32000000;nch=4;okflag=1;}
  else if(tmp.find("16Msps/2bit/04ch")!= tmp.npos) {srate=16000000;nch=4;okflag=1;}
  else if(tmp.find("08Msps/2bit/04ch")!= tmp.npos) {srate= 8000000;nch=4;okflag=1;}
  else if(tmp.find("04Msps/2bit/04ch")!= tmp.npos) {srate= 4000000;nch=4;okflag=1;}
  else if(tmp.find("64Msps/2bit/02ch")!= tmp.npos) {srate= 64000000;nch=2;okflag=1;}
  else if(tmp.find("32Msps/2bit/02ch")!= tmp.npos) {srate= 32000000;nch=2;okflag=1;}
  else if(tmp.find("16Msps/2bit/02ch")!= tmp.npos) {srate= 16000000;nch=2;okflag=1;}
  else if(tmp.find("08Msps/2bit/02ch")!= tmp.npos) {srate= 8000000;nch=2;okflag=1;}
  else if(tmp.find("04Msps/2bit/02ch")!= tmp.npos) {srate= 4000000;nch=2;okflag=1;}
  else{    okflag=0; nch=0; srate=0;  }

//   if(tmp.find("64Msps/2bit/16ch")!= tmp.npos) {speed=64000000;srate=64000000;nch=16;okflag=1;}
//   if(tmp.find("32Msps/2bit/16ch")!= tmp.npos) {speed=32000000;srate=32000000;nch=16;okflag=1;}
//   if(tmp.find("16Msps/2bit/16ch")!= tmp.npos) {speed=16000000;srate=16000000;nch=16;okflag=1;}
//   if(tmp.find("08Msps/2bit/16ch")!= tmp.npos) {speed= 8000000;srate= 8000000;nch=16;okflag=1;}
//   if(tmp.find("04Msps/2bit/16ch")!= tmp.npos) {speed= 4000000;srate= 4000000;nch=16;okflag=1;}
//   if(tmp.find("64Msps/2bit/08ch")!= tmp.npos) {speed=32000000;srate=64000000;nch=8;okflag=1;}
//   if(tmp.find("32Msps/2bit/08ch")!= tmp.npos) {speed=16000000;srate=32000000;nch=8;okflag=1;}
//   if(tmp.find("16Msps/2bit/08ch")!= tmp.npos) {speed= 8000000;srate=16000000;nch=8;okflag=1;}
//   if(tmp.find("08Msps/2bit/08ch")!= tmp.npos) {speed= 4000000;srate= 8000000;nch=8;okflag=1;}
//   if(tmp.find("04Msps/2bit/08ch")!= tmp.npos) {speed= 2000000;srate= 4000000;nch=8;okflag=1;}
//   if(tmp.find("64Msps/2bit/04ch")!= tmp.npos) {speed=16000000;srate=64000000;nch=8;okflag=1;}
//   if(tmp.find("32Msps/2bit/04ch")!= tmp.npos) {speed= 8000000;srate=32000000;nch=4;okflag=1;}
//   if(tmp.find("16Msps/2bit/04ch")!= tmp.npos) {speed= 4000000;srate=16000000;nch=4;okflag=1;}
//   if(tmp.find("08Msps/2bit/04ch")!= tmp.npos) {speed= 2000000;srate= 8000000;nch=4;okflag=1;}
//   if(tmp.find("04Msps/2bit/04ch")!= tmp.npos) {speed= 1000000;srate= 4000000;nch=4;okflag=1;}
//   if(tmp.find("64Msps/2bit/02ch")!= tmp.npos) {speed= 8000000;srate= 64000000;nch=2;okflag=1;}
//   if(tmp.find("32Msps/2bit/02ch")!= tmp.npos) {speed= 4000000;srate= 32000000;nch=2;okflag=1;}
//   if(tmp.find("16Msps/2bit/02ch")!= tmp.npos) {speed= 2000000;srate= 16000000;nch=2;okflag=1;}
//   if(tmp.find("08Msps/2bit/02ch")!= tmp.npos) {speed= 1000000;srate= 8000000;nch=2;okflag=1;}
//   if(tmp.find("04Msps/2bit/02ch")!= tmp.npos) {speed=  500000;srate= 4000000;nch=2;okflag=1;}

  if(okflag){
    vlbi_frame->Set_Sparam(nch,2,srate);
    return 0;
  }else{
    cerr << "W K5_VSI::Set_Mode(): Bad mode option " << tmp << endl;
    return -1;
  }
  
}

string K5_VSI::Mode(){
  if( vlbi_frame == 0 ){
    cerr << "Error in K5_VSI::Set_Nbit(): *vlbi_frame is empty." << endl;
    return "";
  }
  return vlbi_frame->DAS_Inf().Mode();
} ///< Return Obseravtion mode

/** Inversely set */
// const int K5_VSI::Set_Speed(){
//   if( vlbi_frame == 0 ){
//     cerr << "Error in K5_VSI::ISet_Mode(): *vlbi_frame is empty." << endl;
//     return -1;
//   }

//   speed = vlbi_frame->DAS_Info().Speed();

//   cerr << "ISet_Mode():: mode=" << vlbi_frame->DAS_Info().Mode(1) << endl;
//   return ret;

// }
/// Mapping data array to the memory
const uint8_t* K5_VSI::MappingData(time_t epoch){

  if(device ==0){
    cerr << "Error @K5_VSI::MappingData(): K5VSI Device is not yet opened\n";
    return 0;
  }
  if(! map_enable){
    cerr << "Warning @K5_VSI::MappingData(): Map Disabled\n";
    return 0;
  }

  time_t t_now(0);
  rpage = epoch % npage;
  int found=0;
  char time_code[2][80];

  while(found ==0){
    t_now = ioctl(device,GET_FRAME_EPOCH, rpage);
    if(t_now < epoch){
      int tdif = epoch - t_now;
      strftime(time_code[0],sizeof(time_code),"%Y/%j %H:%M:%S", gmtime(&epoch));
      strftime(time_code[1],sizeof(time_code),"%Y/%j %H:%M:%S", gmtime(&t_now));
      if(tdif > 10){
// 	strftime(time_code[0],sizeof(time_code),"%Y/%j %H:%M:%S", gmtime(&epoch));
// 	strftime(time_code[1],sizeof(time_code),"%Y/%j %H:%M:%S", gmtime(&t_now));
	fprintf(stderr,
		"Warning in K5_VSI::MappingData(): expecting epoch(%s) >tnow(%s) more than 10sec.\n",
		time_code[0], time_code[1]);
      }
      fprintf(stderr,"Waiting in K5VSI::MappingData() %s --now=%s, Tdif= %d sec\n", 
	      time_code[0], time_code[1], tdif);
      sleep(1); 
      continue;
    }
    
    if(t_now > epoch){
      strftime(time_code[0],sizeof(time_code),"%Y/%j %H:%M:%S", gmtime(&epoch));
      strftime(time_code[1],sizeof(time_code),"%Y/%j %H:%M:%S", gmtime(&t_now));
      cerr << "Error in K5_VSI::MappingData(): expecting epoch("
	   << time_code[0] <<  ")  is already past tnow(" << time_code[1] << ")\n";
      return 0;
    } 

    found =1;
  }
  
  // page_length is page_size(1sec) in 4-byte unit 
  uint32_t page_length=(unsigned int)ioctl(device,GET_FRAME_CLOCK ,rpage);
  // Get top address of that page
  uint32_t page_offset=(unsigned int)ioctl(device,GET_FRAME_OFFSET,rpage);
  
  // Mapping page to memory
  page_ptr =(uint8_t *)mmap(0x0,sizeof(unsigned int)*page_length,
			    PROT_READ,MAP_SHARED,mem,page_offset);
  return page_ptr;
  
}

// const int K5_VSI_SENDER::Setup_Net(){
//   int ret(0);
//   if(! outfile.empty()){
// //     ret = fout_sock_->wr_open(outfile);    
// //     if(ret <0){
// //       fprintf(stderr, "Failed to open file(%s)\n", outfile.c_str());
// //       delete fout_sock_; fout_sock_=NULL;
// //     }
//     if((ofp = fopen(outfile.c_str(), "w"))==NULL){
//       fprintf(stderr, "Failed to open file(%s)\n", outfile.c_str());
//       ofp=NULL;
//     }
//   }
//   Sock_Create();
//   ret = sock_->connect(hostname,data_port);    
//   if(ret <0){
//     cerr << "  Error in Setup_Net(): connect to host(" << hostname << ":" 
// 	 << data_port << ") failed\n";
//   }
//   return(ret);
// }



// No inversion
int make_tab_ni(unsigned char table[256])
{

  int index;
  memset(table,0,256);
  for(index=0;index<256;index++){table[index]= index;  }

  fprintf(stderr,"Bit encoding No change\n");
  return 0;
}

/// Inverting 2 bit
/// This makes nomal offset-binary codeing, because  
/// ADS-2000 output msb/lsb is opposite it its 2 bit sampling.
int make_tab(unsigned char table[256])
{
  int index;
  fprintf(stderr,"Bit encoding OffsetBin\n");
  memset(table,0,256);
  for(index=0;index<256;index++){
    if(index&0x80) table[index]|=0x40;
    if(index&0x40) table[index]|=0x80;
    if(index&0x20) table[index]|=0x10;
    if(index&0x10) table[index]|=0x20;
    if(index&0x08) table[index]|=0x04;
    if(index&0x04) table[index]|=0x08;
    if(index&0x02) table[index]|=0x01;
    if(index&0x01) table[index]|=0x02;
  }
  return 0;
}

int make_tab_sgnmag(unsigned char table[256])
{
  int index;
  fprintf(stderr,"Bit encoding SignManitude\n");

  memset(table,0,256);

  for(index=0;index<256;index++){
    if(index&0x80) table[index]|=0x40;
    if(index&0x40) table[index]|=0x80;
    if(index&0x20) table[index]|=0x10;
    if(index&0x10) table[index]|=0x20;
    if(index&0x08) table[index]|=0x04;
    if(index&0x04) table[index]|=0x08;
    if(index&0x02) table[index]|=0x01;
    if(index&0x01) table[index]|=0x02;
  }
  
  for(int i=0;i<256;i++){
    if( (table[i] & 0x80) == 0) table[i] ^= 0x40; 
    if( (table[i] & 0x20) == 0) table[i] ^= 0x10; 
    if( (table[i] & 0x08) == 0) table[i] ^= 0x04; 
    if( (table[i] & 0x02) == 0) table[i] ^= 0x01; 
  }
  
  return 0;
}

int32_t K5_VSI::Convert(vector <uint8_t> & arr, int iofst){

  //  int32_t rq_Byte_sec = srate/8*nch*nbit;
  int32_t rq_Byte_sec = vlbi_frame->DAS_Inf().Speed()*4;
  int32_t srate = vlbi_frame->DAS_Inf().Srate();
  if( arr.size() < (uint32_t)rq_Byte_sec ) arr.resize(rq_Byte_sec + iofst);
  
  uint32_t page_length = PageLength(0); // page size in words (4bytes)
  
  int skip=page_length/srate; // ratio of acquisition-rate/output rate   
  if(skip<1){
    fprintf(stderr,"Error: Pagelength(%d)/srate(%d) <1 \n",
	    page_length, srate);
    return  -1;
  }
  int ok_flag=0;
  vector <uint8_t>::iterator ptr = arr.begin() + iofst;
  int32_t ibyte(0);
  string mode = vlbi_frame->DAS_Inf().Mode();

  if(mode.find("/2bit/16ch") < mode.npos){
    for( ibyte=0;ibyte<rq_Byte_sec && ptr < arr.end();ibyte+=4){
      *ptr =enc_table[page_ptr[1*skip*(ibyte)+0]]; ptr++;
      *ptr =enc_table[page_ptr[1*skip*(ibyte)+1]]; ptr++;
      *ptr =enc_table[page_ptr[1*skip*(ibyte)+2]]; ptr++;
      *ptr =enc_table[page_ptr[1*skip*(ibyte)+3]]; ptr++;
      //       arr[iofst+ibyte + 0]=enc_table[page_ptr[1*skip*(ibyte)+0]];
//       arr[iofst+ibyte + 1]=enc_table[page_ptr[1*skip*(ibyte)+1]];
//       arr[iofst+ibyte + 2]=enc_table[page_ptr[1*skip*(ibyte)+2]];
//       arr[iofst+ibyte + 3]=enc_table[page_ptr[1*skip*(ibyte)+3]];
    } ok_flag=1;
  } 
  if(mode.find("/2bit/08ch")< mode.npos){  
    for(ibyte=0;ibyte<rq_Byte_sec && ptr < arr.end();ibyte+=4){
      *ptr = enc_table[page_ptr[2*skip*(ibyte+0)+0+igrp*2]]; ptr++;
      *ptr = enc_table[page_ptr[2*skip*(ibyte+0)+1+igrp*2]]; ptr++;
      *ptr = enc_table[page_ptr[2*skip*(ibyte+2)+0+igrp*2]]; ptr++;
      *ptr = enc_table[page_ptr[2*skip*(ibyte+2)+1+igrp*2]]; ptr++;
//       arr[iofst+ibyte + 0]=enc_table[page_ptr[2*skip*(ibyte+0)+0+igrp*2]];
//       arr[iofst+ibyte + 1]=enc_table[page_ptr[2*skip*(ibyte+0)+1+igrp*2]];
//       arr[iofst+ibyte + 2]=enc_table[page_ptr[2*skip*(ibyte+2)+0+igrp*2]];
//       arr[iofst+ibyte + 3]=enc_table[page_ptr[2*skip*(ibyte+2)+1+igrp*2]];
    } ok_flag=1;
  }
  if(mode.find("/2bit/04ch")< mode.npos){
    for(ibyte=0;ibyte<rq_Byte_sec && ptr < arr.end();ibyte+=4){
      *ptr = enc_table[page_ptr[4*skip*(ibyte+0)+0+igrp]]; ptr++;
      *ptr = enc_table[page_ptr[4*skip*(ibyte+1)+0+igrp]]; ptr++;
      *ptr = enc_table[page_ptr[4*skip*(ibyte+2)+0+igrp]]; ptr++;
      *ptr = enc_table[page_ptr[4*skip*(ibyte+3)+0+igrp]]; ptr++;
//       arr[iofst+ibyte + 0]=enc_table[page_ptr[4*skip*(ibyte+0)+0+igrp]];
//       arr[iofst+ibyte + 1]=enc_table[page_ptr[4*skip*(ibyte+1)+0+igrp]];
//       arr[iofst+ibyte + 2]=enc_table[page_ptr[4*skip*(ibyte+2)+0+igrp]];
//       arr[iofst+ibyte + 3]=enc_table[page_ptr[4*skip*(ibyte+3)+0+igrp]];
    } ok_flag=1;
  }
  
  if(mode.find("/2bit/02ch")<mode.npos){
    for(ibyte=0;ibyte<rq_Byte_sec && ptr < arr.end();ibyte+=4){
      *ptr = enc_table[    (page_ptr[8*skip*(ibyte)+0]&0x0f)
		       |16*(page_ptr[8*skip*(ibyte)+4]&0x0f)]; ptr++;
      *ptr = enc_table[    (page_ptr[8*skip*(ibyte)+0]&0x0f)
		       |16*(page_ptr[8*skip*(ibyte)+12]&0x0f)]; ptr++;
      *ptr = enc_table[    (page_ptr[8*skip*(ibyte)+0]&0x0f)
		       |16*(page_ptr[8*skip*(ibyte)+20]&0x0f)];  ptr++;
      *ptr = enc_table[    (page_ptr[8*skip*(ibyte)+0]&0x0f)
		       |16*(page_ptr[8*skip*(ibyte)+28]&0x0f)];  ptr++;
//       arr[iofst+ibyte+0]=
// 	table[    (page_ptr[8*skip*(ibyte)+0]&0x0f)
// 		  |16*(page_ptr[8*skip*(ibyte)+4]&0x0f)];
//       arr[iofst+ibyte+1]=
// 	table[    (page_ptr[8*skip*(ibyte)+8]&0x0f)
// 		  |16*(page_ptr[8*skip*(ibyte)+12]&0x0f)];
//       arr[iofst+ibyte+2]=
// 	  table[    (page_ptr[8*skip*(ibyte)+16]&0x0f)
// 		    |16*(page_ptr[8*skip*(ibyte)+20]&0x0f)];
//       arr[iofst+ibyte+3]=
// 	table[    (page_ptr[8*skip*(ibyte)+24]&0x0f)
// 		  |16*(page_ptr[8*skip*(ibyte)+28]&0x0f)];
    } ok_flag=1;
  }

  if(ok_flag==0){
    cerr << "Error in K5_VSI::Convert(): mode(" << mode << ") is not supported\n";
    return -1;
  }
  if( ibyte < rq_Byte_sec){
    cerr << "Error in K5_VSI::Convert(): End of array before converting all data\n";
    cerr << "\tConverted =" << ibyte << "Byte, requested =" << rq_Byte_sec
	 << "Byte, arr.size=" << arr.size()  << " Bytes.\n";
    return -1;
  }
  return ibyte+iofst;
}

const K5_VSI::IndevReturns_t  K5_VSI::UnMapping(const time_t epoch){
    if(munmap(page_ptr,4*PageLength(rpage)) !=0){
      perror("Error in K5_VSI::UnMapping()");
      return GEN_ERROR;
    }
    return NORMAL;
}


int K5_VSI::FirstPage(){
  time_t tmin=0, twork;
  for(int i=0; i<npage; i++){
    if( (twork = ioctl(device,GET_FRAME_EPOCH,i)) > 0){
      if(tmin == 0 && twork > 0) tmin = twork;
      if(tmin > twork) tmin = twork;
    }else{
      cerr << "Error @K5_VSI::FirstPage():Failed to get epoch(" << i << ")" 
	   << strerror(errno) << endl;
    }
  }
  int ipg = (tmin + npage/2) % npage;
  return ipg;
}

// const K5_VSI::IndevReturns_t 
// K5_VSI::MappingData(vector <uint8_t>& arr,time_t epoch){
//   cerr << "Error @MappingData(): This is dummy function in K5_VSI."
//        <<" Use other MappingData() instead." << endl;
//   return GEN_ERROR;
// }; 



