/**  Transmission information class
 * $Id: Trans_info.hpp,v 1.2 2010/01/02 03:29:17 sekido Exp $
 */
#ifndef __TRANS_INFO_HPP__
#define __TRANS_INFO_HPP__


#include <string>
#include <iostream>
using namespace std;

extern "C"{
#include <stdint.h>  
}

const int DEFAULT_DATA_PORT  = 9901;
const int DEFAULT_SUDP_PORT  = 9901;
//const int DEFAULT_CTRL_PORT = 46224;
const int DEFAULT_CTRL_PORT  = 9902;  // used for sender
const int DEFAULT_CTRL_PORT2 = 9903;  // used for receiver

const int DEFAULT_MTU_SIZE  = 1500;
const int TCPIP_HD_SIZE = 48; 
const int UDPIP_HD_SIZE = 32;
const int SUDP_HEAD_SIZE   =  8; // (bytes)

/**
 * A Class of Transmissin information 
 */

class TransInfo{
public:
  /// Transport Layer protocol types
  enum TransProtocol_t {TCP_IO, UDP_IO, FILE_IO};
  /// Application protocol types
  enum ApplProtocol_t {VDIF_Proto, SUDP_Proto, RAW_Proto};
  /// Application Protocol states
  enum ApplProtocolState_t {IDLE, RX_READY, RXING, TX_READY, TXING, STOP};
  /// Function mode type 
  enum Mode_t {TX, RX};
  //  enum Speed_flag_t {USING_DASINF, USING_TRNSINF};
  /// Constructor
  TransInfo(const ApplProtocol_t apt=SUDP_Proto, 
	    const TransProtocol_t tpt=UDP_IO); 
  TransInfo(const int m); ///< Constructor
  /** Set Transmission Layer Protocol */
  void Set_Transprotocol(const TransProtocol_t p){ transLayerProtocol=p;}
  /** Set Application Layer Protocol */
  void Set_Applprotocol(const ApplProtocol_t p){ applLayerProtocol=p;}
  /** Return Transmission Layer Protocol */
  TransProtocol_t TransProtocol() const {return transLayerProtocol;}
  /// Returns type of transport portocol in string 
  string TransProtocol();
  /** Return Application Layer Protocol */
  ApplProtocol_t ApplProtocol() const {return applLayerProtocol;}
  /// Set Destination host name
  void Set_Host(const char* h){hostname=h;}
  /// Set Data port number
  void Set_DataPort(const int p) {data_port=p;}
  string Host(){return hostname;} ///< Returns remote destination Hostname
  int DataPort() const {return data_port;} ///< Return Data Port number
  /**  Set Transmission length(sec) */ 
  void Set_TransLength(const uint32_t sec){ txrx_length = sec;}
  void Set_OprMode(Mode_t m) {opr_mode=m;} ///< Set operation mode (TX/RX)
  Mode_t Mode_opr()const { return opr_mode;} ///< Operation mode
  string Status(); ///< Return Status Info
  int  Set_mtu(const int m); ///< Set MTU size (byte)
  /// Returns MTU size (Bytes)
  const int Mtu()const {return mtu;} ///< Return MTU (byte)
  /// Set Speed paramter
  void Set_Speed(const int wordps, const bool use_txspeed){
    speed = wordps; if(use_txspeed) useTransSpeed = true;}
  /// Returns the flag of using this 'speed' parameter or das_info
  const bool UseTransInfSpeed() const { return (useTransSpeed);}
  /// Returns Tx speed  (word/sec) in this class./
  const int32_t Speed()const {return speed;} // word/sec
  /// Returns State of Application Protocol (e.g. SUDP) 
  const ApplProtocolState_t ProtocolState() const {return applProtocolState;} 
  /// Stop running protocol
  void Protocol_stop(){
    applProtocolState = STOP;  
    cerr << "I Trans_Info::Protocol_stop(): => stop" << endl; }
protected:
  int mtu; ///< MTU size (byte)
  int32_t speed;  ///< TX Rate words/sec(default:2000000 = 64Mbps)
  int data_port;  ///< Data port
  TransProtocol_t transLayerProtocol; ///< Transmission Protocl
  ApplProtocol_t  applLayerProtocol; ///< Applycation Protocl
  ApplProtocolState_t   applProtocolState; ///< Current working status 
  string hostname;    ///< remote destination hostname
  int32_t txrx_length;///< Receiving/Transmission length of data (sec) 
  time_t  EndEpoch;   ///< TxRx End Epoch 0:endless, 
  Mode_t opr_mode;    ///< Operation mode (TX/RX)
  //  Speed_flag_t sflag;
  bool useTransSpeed; ///< Flag for Tx speed paramete or das_inf data rate. 
};


#endif
