
#include "VLBI_Frame.hpp"


VDIF_Frame::VDIF_Frame():VLBI_Frame(DEFAULT_MTU_SIZE){
    Time_dat t1(2000,1,1,0,0,0);
    ref_epoch0 = t1;
    vdif_hdr.Set_Nch(das_inf.Nch());
    vdif_hdr.Set_Qbit(das_inf.Qbit());
    vdif_hdr.Set_Srate(das_inf.Srate()/1000);
}


/** Set Sampling parameters, s:(Hz),  return data rate (Bytes/sec)  */
int32_t 
VLBI_Frame::Set_Sparam(const int n, const int q, const uint32_t s){
//   if(das_inf==0){
//     cerr << "Error in VLBI_Frame::Set_Param(): DAS_Inf is not set \n";
//     return -1;
//   }
  das_inf.Set_Param(n,q,s); 
  return das_inf.Speed()*4; 
}
  
int Mk5B_Frame::Set_header(const uint8_t* buf, const int ifrm){
  m5hdr.Set_header(data_epoch);
  unsigned char crcdata[6];

  crcdata[0] = m5hdr.mjd2<<4|m5hdr.mjd1;
  crcdata[1] = m5hdr.mjd0<<4|m5hdr.s4;
  crcdata[2] = m5hdr.s3<<4  |m5hdr.s2;
  crcdata[3] = m5hdr.s1<<4  |m5hdr.s0;
 
  crcdata[4] = m5hdr.ms4<<4|m5hdr.ms3;
  crcdata[5] = m5hdr.ms2<<4|m5hdr.ms1;
  m5hdr.crc = pCRC_mdl->Cmp_CRC(crcdata,6);
  if( ifrm >=0)  
    m5hdr.iframe = ifrm;
  else
    m5hdr.iframe++;
  memcpy((void*)buf,&m5hdr, Mk5B_Hdr_size);
  //  memcpy((void*)buf,&m5hdr, sizeof(m5hdr));
  return sizeof(m5hdr);
}

// void Mk5B_Frame::Set_mark5header( uint32_t *header, const time_t epoch, 
// 				  const double sec, 
// 				  const uint16_t iblk){
//   header[0]=0xABADDEED;
//   header[1]=0xF00F0000 | iblk;
//   header[2]=0x00000000;
//   header[3]=0x00000000;
//   //  uint16_t *crc = (unsigned short*)&header[3];


//   //  char time_code[32];
//   //  strftime(time_code,sizeof(time_code),"%Y/%j %H:%M:%S",gmtime(&epoch));
//   // strptime(time_code,"%Y/%j %H:%M:%S",&tm);
//   //  double mjd = tm2mjd(tm);
//   //  fprintf(stderr, "MJD1 = %.5lf ",mjd);

//   struct tm tm;
//   gmtime_r(&epoch,&tm);
//   double mjd=tm2mjd(tm);
//   //  fprintf(stderr, "MJD2 = %.5lf \n",mjd);


//   int intmjd = (int)floor(mjd);
//   int secmjd = (int)floor((mjd-intmjd)*86400+5e-7);  // Avoid round off errors
  
//   //  fprintf(stderr,"Writing: %03d/%04d +%6.4lf sec \n", intmjd);
//   // Word 2 VLBA BCD time code 'JJJSSSSS'. Only needs to be updated
//   // once a second

//   unsigned char j1,j2,j3,s1,s2,s3,s4,s5;  
//   j1 = (intmjd%1000/100);
//   j2 = ((intmjd%100)/10);
//   j3 = (intmjd%10);
  
  
//   s1 = (secmjd/10000)&0xF;
//   s2 = ((secmjd%10000)/1000)&0xF;
//   s3 = ((secmjd%1000)/100)&0xF;
//   s4 = ((secmjd%100)/10)&0xF;
//   s5 = (secmjd%10)&0xF;
  
//   header[2]  = j1<<28 | j2<<24 | j3<<20 | s1<<16 | s2<<12 | s3<<8 | s4<<4 | s5;
  
//   unsigned char crcdata[6];
//   crcdata[0] = j1<<4|j2;
//   crcdata[1] = j3<<4|s1;
//   crcdata[2] = s2<<4|s3;
//   crcdata[3] = s4<<4|s5;
  
//   // MJD sec fraction
//   int fracsec = (int)(sec*10000);
  
//   s1 = fracsec/1000;
//   s2 = (fracsec%1000)/100;
//   s3 = (fracsec%100)/10;
//   s4 = fracsec%10;
//   header[3] = s1<<28 | s2<<24 | s3<<20 | s4<<16;
  
//   crcdata[4] = s1<<4|s2;
//   crcdata[5] = s3<<4|s4;

//   uint16_t crc = pCRC_mdl->Cmp_CRC(crcdata,6);
//   // *crc = reversebits16(crcc(crcdata, 48, 040003, 16));
//   //  printf("Header = {%08x,%08x,%08x,%08x}\n",header[0],header[1],header[2],header[3]);

//   header[3] |= (crc & 0xffff);
//   //  printf("Header = {%08x,%08x,%08x,%08x}\n",header[0],header[1],header[2],header[3]);
// }  

/// ################# VSSP32 From here 
/** Set Sampling parameters, s:(Hz),  return data rate (Bytes/sec)  */
int32_t K5VSSP32_Frame::Set_Sparam(const int n, const int q, const uint32_t s){
  VLBI_Frame::Set_Sparam(n,q,s);
  k5hdr.Set_Qbit(q);
  k5hdr.Set_Srate(s);
  if(k5hdr.Set_Nch(n)<0){  return -1;  }
  framesize = k5hdr.Framesize();
  return das_inf.Speed()*4;
}

void K5VSSP32_Frame::Set_Epoch(const Time_dat& t1){
  data_epoch=t1;
  k5hdr.Set_Epoch(t1);
}

void K5VSSP32_Frame::Set_Epoch(const time_t t1){
  struct tm tm1;
  //  time_t t2 =t1;
  gmtime_r(&t1, &tm1);
  int yy = tm1.tm_year + 1900;
  int mon = tm1.tm_mon + 1;
  int dd = tm1.tm_mday;
  int hh = tm1.tm_hour;
  int mm = tm1.tm_min;
  int ss = tm1.tm_sec;
  Time_dat td(yy,mon,dd,hh,mm,ss);
  Set_Epoch(td);
}

int  K5VSSP32_Frame::Set_header(const uint8_t* buf, const int ifrm){
  K5_Aux_1* pk5aux = (K5_Aux_1*)k5hdr.ext_aux;
  pk5aux->aux_fmt_No = 1;
  //  k5hdr.Set_Stname(stname);
  //  k5hdr.Set_StnID(id);  
  memcpy(pk5aux->station_id, stid.c_str(),2);
  memcpy(pk5aux->stname,stname.c_str(),8);
  char host[80];
  gethostname(host,80);
  int len = strlen(host);
  len = len > 8 ? 8 :len;
  memcpy(pk5aux->hostPCname,host,len);
  //  memcpy((void*)buf,&k5hdr, sizeof(k5hdr));
  memcpy((void*)buf,&k5hdr, K5VSSP32_Hdr_size);
  return sizeof(k5hdr);
}


const int K5VSSP32_Frame::Set_DasInfo(const DAS_Info& ds){
  das_inf = ds;
  try{
    if( k5hdr.Set_Qbit(ds.Qbit()) < 0 ) throw -1;
    if( k5hdr.Set_Srate(ds.Srate()) < 0 ) throw -1;
    if( k5hdr.Set_Nch(ds.Nch()) <0 ) throw -1;
  }
  catch(const int ret){
    if(ret < 0){
      cerr << "Error at K5VSSP32_Frame::Set_DasInfo(): Incompatible DAS parameters:"  << ds.Mode() << endl;
      return -1;
    }
  }
  framesize = k5hdr.Framesize();
  return 0;
}

/// ################# VSSP32 Until here 

/// ################# Misclanious tools 
double tm2mjd(struct tm date) {
  int m, y, c, x1, x2, x3;
  double dayfrac;

  if (date.tm_mon < 2) {
    m = date.tm_mon+10;
    y = date.tm_mon+1900-1;
  } else {
    m = date.tm_mon-2;
    y = date.tm_year+1900;
  }

  c = y/100;
  y = y-c*100;

  x1 = 146097*c/4;
  x2 = 1461*y/4;
  x3 = (153*m+2)/5;

  dayfrac = ((date.tm_hour*60.0+date.tm_min)*60.0+date.tm_sec)
    /(60.0*60.0*24.0);

  return(cal2mjd(date.tm_mday, date.tm_mon+1, date.tm_year+1900)
	 +dayfrac);
}


// void VLBI_Frame::Set_Epoch(const time_t t1){
//   if(ref_epoch0.Mjd() ==0) 
//     ref_epoch0 = vdif_hdr.Set_Epoch0(t1);

//   data_epoch = t1; 
//   vdif_hdr.Set_Epoch(t1);
// }


double cal2mjd(int day, int month, int year) {
  int m, y, c, x1, x2, x3;

  if (month <= 2) {
    m = month+9;
    y = year-1;
  } else {
    m = month-3;
    y = year;
  }

  c = y/100;
  y = y-c*100;

  x1 = 146097*c/4;
  x2 = 1461*y/4;
  x3 = (153*m+2)/5;

  return(x1+x2+x3+day-678882);
}


