
#include "VsiData.hpp"
#include <cstring>

const int VsiData::Make_Table(){
  if(dasInfo ==0){
    cerr << "Error @VsiData::Make_Table(): dasInfo is empty.\n";
    return -1;
  }
  try{
    switch(dasInfo->Qbit()){
    case 1: 	MkTbl1bit();	break;
    case 2: 	MkTbl2bit();	break;
    case 4: 	MkTbl4bit();	break;
    case 8: 	MkTbl8bit();	break;
    default:	throw "Impossible Qbit"; break;
    }
  }

  catch(const char* msg){
    cerr << "Error @VsiData::Make_Table(): " << msg << endl;
    return -1;
  }
  return 0;
}


const int VsiData::MkTbl1bit(){
  if(dasInfo ==0){  
    cerr << "Error @VsiData::MkTbl4_1(): dasInfo is empty.\n";
    return -1;  
  }
    for(int i=0; i<256; i++)  memset(table[i], 0, MAX_NCH);
  int8_t *ptr;
  for(int i=0; i<256; i++){
    ptr = &table[i][0];
    if(i & 0x01)  *ptr=level_1bit[1]; else *ptr=level_1bit[0]; ptr++;
    if(i & 0x02)  *ptr=level_1bit[1]; else *ptr=level_1bit[0]; ptr++;
    if(i & 0x04)  *ptr=level_1bit[1]; else *ptr=level_1bit[0]; ptr++;
    if(i & 0x08)  *ptr=level_1bit[1]; else *ptr=level_1bit[0]; ptr++;
    if(i & 0x10)  *ptr=level_1bit[1]; else *ptr=level_1bit[0]; ptr++;
    if(i & 0x20)  *ptr=level_1bit[1]; else *ptr=level_1bit[0]; ptr++;
    if(i & 0x40)  *ptr=level_1bit[1]; else *ptr=level_1bit[0]; ptr++;
    if(i & 0x80)  *ptr=level_1bit[1]; else *ptr=level_1bit[0]; 
  }
  return 0;
}

const int VsiData::MkTbl2bit(){
  if(dasInfo ==0){  
    cerr << "Error @VsiData::MkTbl4_1(): dasInfo is empty.\n";
    return -1;  
  }
  for(int i=0; i<256; i++)  memset(table[i],0, MAX_NCH*sizeof(uint8_t));

  uint8_t mask[4];
  mask[0] = 0x03;
  mask[1] = 0x03 << 2;
  mask[2] = 0x03 << 4;
  mask[3] = 0x03 << 6;

  for(int i=0; i<256; i++){
    for(int j=0; j<4; j++){
      int v = (i & (0x03 << (j*2))) >> (j*2);
      table[i][j] = level_2bit[v];	
    };
  }
  return 0;
}


const int VsiData::MkTbl4bit(){
  if(dasInfo ==0){  
    cerr << "Error @VsiData::MkTbl4_1(): dasInfo is empty.\n";
    return -1;  
  }
  for(int i=0; i<256; i++)  memset(table[i],0, MAX_NCH*sizeof(uint8_t));
  for(int i=0; i<256; i++){
    table[i][0] = i & 0x0f - 8;
    table[i][1] = ((i & 0xf0) >> 4) - 8;
  }
  return 0;
}

const int  VsiData::MkTbl8bit(){
  if(dasInfo ==0){  
    cerr << "Error @VsiData::MkTbl4_1(): dasInfo is empty.\n";
    return -1;  
  }
  for(int i=0; i<256; i++)  memset(table[i],0, MAX_NCH*sizeof(uint8_t));
  for(int i=0; i<256; i++){
    table[i][0] = i -128;
  }
  return 0;
}

const int32_t VsiData::Read1Sec(FILE* fp){
  int32_t ByteSec = dasInfo->Speed()*4;
  if( (int32_t)buf.size() < ByteSec) buf.resize(ByteSec);
  
  if( fread(&buf[0],ByteSec,1,fp) < 1){
    cerr << "Error in Read1Sec()\n";
    return -1;
  }
  return ByteSec;
}

