#include <iostream>
#include <string>
#include <vector>
using namespace std;
#include <boost/algorithm/string.hpp>

#include "SUDPProto.hpp"
#include "Trans_ctl.hpp"
#include "Raw_Proto.hpp"
#include "InNet_mdl.hpp"
#include "Dumper.hpp"
#include "StaticVals.hpp"
#include "LogSet.hpp"

char *me;  // My name

void usage(){
  cerr << "Usage: " << me <<" [-p <ctrl port>] [-P <data port>] [-g <n>] [-v <n>]\n";
  cerr << "    [-m <mode>]   [-i <info logfile>][-x <Norm|Mk5Geo>] \n";
  //  cerr << "   [-m <outmode>] [-M <inmode>] [-l <sec>]\n";
  //  cerr << "  [-r <Mbps>]  [-d <logging file>]  [-a <host>]\n";
  cerr << "  ----------------------------------------------------\n";
  cerr << "  Server mode data receiver of VDIF/SDUP stream\n";
  cerr << " -p <port>, port number for communication with client. default=9903.\n";
  cerr << " -P <port>, port number for receiving data stream. default=9901.\n";
  cerr << " -g: Number of max output threads.\n";
  cerr << " -m: Input data mode. ex.: 32Msps/2bit/16ch is default.\n";
  cerr << " -x: Extraction mode Normal(default) or Mk5Geo(stream 16-19/32 are LSB(14,15ch).\n";
  cerr << " -v: verbose level, 0--4 (default -1: no logging).\n";
  cerr << " -i <info file>: Information(packet-loss etc.) Logging.\n";
  cerr << "-------------------------------------------------------\n";
  cerr << " Supporting format: VDIF, VSSP32. $Id: sudp_recv3.cpp,v 1.8 2011/07/17 07:20:15 vlbi Exp $\n";
  cerr << "-------------------------------------------------------" << endl;
  exit(1);
  //  cerr << " -o <file>[:<file2>..] Output to files for multiple out.\n";
//   cerr << " -t <FMT>: Writing format, VDIF | VSSP. (default:VSSP32).\n";
//   cerr << " -n <nch>, Number of channels (1|4) for output. default=4.\n";
//   cerr << " -g <group>[:<grp>...], Writing Group number of nch. 0,1,2..\n";
//   cerr << " -m <'xxMsps/xbit/xxch'>: Output Data mode\n";
//   cerr << " -M <'xxMsps/xbit/xxch'>: Input  Data mode\n";
//   cerr <<"        Msps -> [04Msps|08Msps|16Msps|32Msps|64Msps]\n";
//   cerr << "        bit  -> [1bit|2bit|4bit|8bit] \n";
//   cerr << "        ch   -> [16ch|08ch|04ch|02ch|01ch] \n";
//   cerr << " -m <mode> Recording mode. ex) 32Msps/2bit/16ch, Default:16Msps/1bit/4ch.\n";
//  cerr << " -s <byte> Packet size (default=1500).\n";
  //  cerr << " -r <Mbps> Expected rate (default=100).\n";
  //cerr << " -l <sec> length of receiving data(default=0 means endless).\n";
}


int main(int argc, char *argv[]){
  me = (me = strrchr(argv[0], '/')) == NULL ? argv[0] : me+1; /* My name */ 
  int rxport(DEFAULT_DATA_PORT);
  int ctrlport(DEFAULT_CTRL_PORT2);
  string ip("0.0.0.0");
  string def_outmode = "08Msps/1bit/4ch";
  LogSet logset(me,-1);
  int ngrp(4) ;// Max number of output groups
  string inmode = "32Msps/2bit/16ch";
  bool mk4geo_mode = false;

  int k=0;
  while (k >= 0) { 
    switch (k = getopt(argc, argv, "hp:P:v:g:m:i:x:")) { 
    case 'p':   ctrlport  = atoi(optarg); break;
    case 'P':   rxport  = atoi(optarg); break;
    case 'm':   inmode = optarg; break;
    case 'i':   logset.Set_infofile(optarg); break;
    case 'v':   logset.Set_level(atoi(optarg)); break;
    case 'g':    ngrp = atoi(optarg); break;
    case 'x':   if(strncmp(optarg,"Mk5Geo",6) == 0) mk4geo_mode=true; break;
    case 'h':    usage(); break;
    case '?':    cerr << "Missing following parameters\n"; usage(); break;
    case ':':    cerr << "Invalid option\n"; usage(); break;
    }
  }

  logset.Start();
  
//   DAS_Info   outdas;
  DAS_Info   indas;
  if( indas.SetMode(inmode)){
    cerr << "Initial Input mode=" << indas.Mode() << endl;
  }else{
    cerr << "Error at mode setting.\n";
    usage();
  };
  
  
//   if(! outdas.SetMode(def_outmode) || ! indas.SetMode(inmode)){
//     cerr << "Error at mode setting\n";
//     usage();
//   };
  
   
  VDIF_Frame in_frame;
  in_frame.Set_DasInfo(indas);

//  cerr << "Intial DAS = " << outdas.Mode() << endl;
  SUDPProto inp;
  inp.Set_VlbiFrame(&in_frame);
  inp.Set_DataPort(rxport);
  //  inp.Set_TransLength(length); // sec
  //  inp.Set_Speed(rate); // (Mbps)
  inp.Disp_RxRate();
  cerr << "Innet Intializing..." << flush;
  InNet_Model innet;
  cerr << " Set Protcol to Innet..." << flush;
  innet.Set_protocol(&inp);
  cerr << " Frame Set..." << flush;
  innet.Set_VlbiFrame(&in_frame);
  //  cerr << " DasInfoe Set..." << flush;
  //  innet.Set_DasInfo(indas);
  cerr << endl;


  /** Rxing Control object */ 
  TransCtl transctl_proto;
  transctl_proto.Set_CtrlPort(ctrlport);

  RAW_Proto outp[ngrp];   // Raw output protocol
  K5VSSP32_Frame out_frame[ngrp];   // VSSP32 output Frame 
  for(int i=0; i<ngrp; i++){
    outp[i].Set_VlbiFrame(&out_frame[i]);
    if(mk4geo_mode){
      outp[i].SetExmode_Mk4Geo(); // Extracting LSB 8,9ch->15,16ch
      cerr << "I: Rcving mode["<< i << "] Mk4VSI-Geodesy" << endl;
    }
  }
  transctl_proto.Set_ApplProto(&outp[0], ngrp); // Set TransInfo
  transctl_proto.Set_InputDev(&innet);
  if(! transctl_proto.SetupCheck()) exit(1);

  transctl_proto.start(); // Transaction server starting

//   while(1){
//     Logsystem::info_logger()->Tstamp()
//       << transctl_proto.DumperState() << "\n";
//     Logsystem::info_logger()->Flush();
//     sleep(1);
//   }

  transctl_proto.join();  // waiting for ending
  cerr << "joined\n";

  inp.Disp_RateOff();  

  logset.Stop();
}
