#!/usr/bin/perl -w

sub usage(){
    print "Usage: $0 -dir <outdir> [-st <yyyydoyhhmmss>] [-m <mode>] \n";
    print "    [-dur <sec>] [-id <stid>] [-nch <nch>] [-h] [-exe]\n";
    print "  -m  : mode xxMsps:xbit:xch [default = 4Msps/1bit/4ch]\n";
    print "  -st : start time. default= now+3 sec\n";
    print "  -dur: recording duration. default=10sec.\n";
    print "  -id : station id. default='O'.\n";
    print "  -nch: Number of Channels(1,4,8,16). default=4.\n";
    print "  -h  : Help display this message.\n";
    print "  -exe: Execute the command.\n";
    print "   ---- \$Id: vssp-rec.pl,v 1.4 2010/01/23 06:13:16 vlbi Exp $ ------\n";
    exit(1);
}

my $cmd0 = "k5vsi-vssp-dim";
use lib "$ENV{HOME}/prog/perl/module";
use Time_lib;
use Term::ANSIScreen qw/:color :cursor :screen :keyboard/;


{
    usage() if(@ARGV < 4);
    my $now = new Time_lib;
    $now->Set_time_now();
    my $t_start = $now + 3;
    my ($odir, $st);
    my $mode = "4Msps:1bit:4ch";
    my $nch  = 4;
    my $dur  = 15;
    my $stid   = 'O';
    my $exflag=0;

    while(@ARGV){
	my $opt = shift;
	if($opt =~/-dir/){   $odir = shift;}
	elsif($opt =~/-st/){  $st = shift;}
	elsif($opt =~/-m/){  $mode = shift;}
	elsif($opt =~/-nch/){  $nch = shift;}
	elsif($opt =~/-id/){  $stid = shift;}
	elsif($opt =~/-dur/){  $dur = shift; $dur +=5;}
	elsif($opt =~/-exe/){  $exflag=1; }
	elsif($opt =~/-h/){  usage();}
    }
    
    usage() unless( defined($odir) );
    if(defined($st)){
	$st =~/(\d{4})(\d{3})(\d\d)(\d\d)(\d\d)/;
	my @ydhms= ($1, $2, $3, $4, $5 );
	$t_start->Set_time_ydhms(@ydhms);
    }

    my $t_end = $t_start + $dur;

    unless($mode =~/:(\d+)ch/){
	print STDERR "E mode($mode) is unexpected format\n";
	usage();
    }
    my $mch = $1;
    unless($nch % $mch == 0){
	print STDERR "E nch($nch) and mode($mode) is inconsistent.\n";
	usage();
    }
    my $nout = $nch/$mch;

    
    if($exflag){
	my   $cons= Term::ANSIScreen->new;
	my $dif = $t_start - $now;
	print  "\n\n";
	while($dif > 0){
	    print up(1);
	    printf "Now(%s) %2d sec until %s\n",
	    $now->Str_fmt3(1), $dif, $t_start->Str_fmt3(1);
	    sleep(1);
	    $now->Set_time_now();
	    $dif = $t_start - $now;
	}
    }


    my @abcd = qw( a b c d);
    $mode =~s!:!/!g;
    my $file = " -o ";
    my $grp = " -g ";
    for(my $i=0; $i<$nout; $i++){
	my $outf = sprintf( "%s/%s%s%s.dat", 
			    $odir, $stid,$t_start->Str_fmt12(), $abcd[$i]);
#			    $odir, $stid,$t_start->Str_fmt12(3), $abcd[$i]);
	if($i == $nout -1) {
	    $file = $file . "$outf";
	    $grp = $grp . "$i";
	}else  {
	    $file = $file . "${outf}:";
	    $grp = $grp . "$i:";
	}
	
    }
    
    my $opt = sprintf (" -t %ds -m $mode $file $grp ", $dur);
    my $cmd = sprintf("%s $opt", $cmd0);
    print STDERR "$cmd\n";
    
    if($exflag){
	if($now < $t_end){
	    system($cmd);
	}else{
	    printf STDERR 
		("E Recording span(%s - %s) has passed now(%s)\n",
		 $t_start->Str_fmt3(1), $t_end->Str_fmt3(1),
		 $now->Str_fmt3(1))	;
	} 
    }
    
    
}

